# Duo, A crazy eights game.
# Copyright (C) 2009 The Duo developers.
#
# This file is part of Duo.
#
# Duo is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""Rulesets for Duo."""


class Rule(object):
    """The referee rules base class.

    Rules build a set of possible actions depending on the conditions
    they receive.
    """
    def __init__(self):
        pass

    def check(self, player, ring, carpet):
        """Check this rule."""
        return set()


class Effect(object):
    def __init__(self, player):
        self.player = player

    def __repr__(self):
        return self.__str__()

    def __str__(self):
        return self.__class__.__name__ + ': ' + str(self.player)

    def apply(self, ring, carpet, bundles, bc_bundle):
        raise NotImplemented, '%s should implement apply' % self.__class__.__name__


def no_referee(fn):
    """Decorator to ignore the Referee (or None player) on rules checking.
    """
    def check_no_referee(self, player, ring, carpet):
        if player is None:
            return []
        else:
            return fn(self, player, ring, carpet)
    return check_no_referee


def only_referee(fn):
    """Decorator to enforce referee only (or None player) on rules checking.
    """
    def check_referee(self, player, ring, carpet):
        if not player is None:
            return []
        else:
            return fn(self, player, ring, carpet)
    return check_referee


def no_kitty(fn):
    """Decorator to enforce the absence of a kitty on rules checking.
    """
    def check_no_kitty(self, player, ring, carpet):
        if len(carpet.kitty) or carpet.kitty.taken:
            return []
        else:
            return fn(self, player, ring, carpet)
    return check_no_kitty


def at_turn(fn):
    """Decorator to enforce the move on turn only.
    """
    def check_at_turn(self, player, ring, carpet):
        if player != ring.current():
            return []
        else:
            return fn(self, player, ring, carpet)
    return check_at_turn
